@@ -5,8 +5,8 @@
 ;; Keywords: multimedia, music, spotify, smudge
 ;; Package: smudge
 ;; Package-Requires: ((emacs "27.1") (simple-httpd "1.5.1") (request "0.3") (oauth2 "0.18"))
-;; Package-Version: 20251224.1549
-;; Package-Revision: dcca1b9ac158
+;; Package-Version: 20260203.20
+;; Package-Revision: c90db830e84e
 ;; Homepage: https://github.com/danielfm/smudge
 
 ;; SPDX-License-Identifier:  GPL-3.0-or-later
@@ -39,6 +39,7 @@
 (require 'smudge-device-select)
 (require 'smudge-controller)
 (require 'smudge-remote)
+(require 'smudge-lyrics)
 
 (smudge-when-darwin    (require 'smudge-apple))
 (smudge-when-gnu-linux (require 'smudge-dbus))
@@ -133,6 +134,7 @@ Prompt for the NAME and whether it should be made PUBLIC."
     (define-key map (kbd "s")   '("smudge/toggle-shuffle" . smudge-controller-toggle-shuffle))
     (define-key map (kbd "p")   '("smudge/playlists"      . smudge-playlists))
     (define-key map (kbd "t")   '("smudge/tracks"         . smudge-tracks))
+    (define-key map (kbd "l")   '("smudge/lyrics"         . smudge-lyrics-popup))
     (define-key map (kbd "v")   '("smudge/volume"         . smudge-volume))
     map)
   "Keymap for Spotify commands after \\='smudge-keymap-prefix\\='.")
@@ -171,6 +173,7 @@ Prompt for the NAME and whether it should be made PUBLIC."
                       ["Play/Pause"     smudge-controller-toggle-play]
                       ["Previous Track" smudge-controller-previous-track]
                       ["Next Track"     smudge-controller-next-track]
+                      ["Lyrics"         smudge-lyrics-popup]
                       "--"
                       ["Select Playing Device" smudge-select-device]
                       ["Mute/Unmute"           smudge-controller-volume-mute-unmute]
@@ -194,6 +197,7 @@ Prompt for the NAME and whether it should be made PUBLIC."
      ["Play/Pause" smudge-controller-toggle-play]
      ["Previous Track" smudge-controller-previous-track]
      ["Next Track" smudge-controller-next-track]
+     ["Lyrics" smudge-lyrics-popup]
      "--"
      ["Select Device" smudge-select-device]
      ["Mute/Unmute" smudge-controller-volume-mute-unmute]
